import 'package:database_interface/database_interface.dart';
import 'package:quiver/core.dart';
import 'package:test/test.dart';

import 'src/fake_no_sql_database.dart';

void main() {
  group('DatabaseIdInterface', () {
    test('hashcode', () {
      final id = DatabaseId('12');
      expect(id.hashCode, hashObjects(<String>['12']));
    });

    test('toString', () {
      final id = DatabaseId('12');
      expect(id.toString(), 'DatabaseIdInterface("12")');
    });

    test('== with a DatabaseIdInterface', () {
      final id1 = DatabaseId('12');
      final id2 = DatabaseId('12');
      expect(id1 == id2, isTrue);
    });

    test('== with a String', () {
      final id1 = DatabaseId('12');
      const Object id2 = '12';
      expect(id1 == id2, isTrue);
    });
  });

  group('DatabaseException', () {
    test('message', () {
      final exception = DatabaseException('Error');
      expect(exception.message, 'Error');
    });
  });

  group('no sql manager', () {
    NoSqlDatabaseInterface fakeNoSqlDatabaseFactory() => FakeNoSqlDatabase();

    test('open', () async {
      final manager = NoSqlDatabaseManager(fakeNoSqlDatabaseFactory);
      final db = await manager.open();
      expect(db is NoSqlDatabaseInterface, isTrue);
      expect(db.isOpen, isTrue);
    });

    test('close', () async {
      final manager = NoSqlDatabaseManager(fakeNoSqlDatabaseFactory);
      final db = await manager.open();
      await manager.close(db);
      expect(db.isOpen, isFalse);
    });
  });

  group('no sql pool', () {
    NoSqlDatabaseInterface fakeNoSqlDatabaseFactory() => FakeNoSqlDatabase();

    test('open', () async {
      final manager = NoSqlDatabaseManager(fakeNoSqlDatabaseFactory);
      final pool = NoSqlDatabasePool.fromManager(manager);
      expect(pool.pool, isNotNull);
      final connection = await pool.pool.get();
      expect(connection.connection.isOpen, isTrue);
      await pool.pool.manager.close(connection.connection);
      expect(connection.connection.isOpen, isFalse);
    });
  });
}
